# Copyright (C) 2010, 2011 Sebastian Thiel (byronimo@gmail.com) and contributors
#
# This module is part of GitDB and is released under
# the New BSD License: http://www.opensource.org/licenses/bsd-license.php
"""Contains library functions"""
from gitdb.test.lib import TestBase



#{ Base Classes

class TestBigRepoR(TestBase):
    """A placeholder in case we want to add additional functionality to all performance test-cases
    """


#} END base classes
